#![allow(clippy::empty_loop)]

use x86_64::instructions::random::RdRand;

use crate::{logger, vterm::Vterm};
use {
    crate::{
        arch::{init, sloop},
        boot_conf,
        boot_conf::BootConfig,
        capabilities::FileAccess,
        experiments::{
            info::master,
            systeminfo::{KERNEL_VERSION, RELEASE_TYPE},
        },
        file::PathRep,
        relib::network::socket::{SimpleSock, Socket},
        scheduler::SCHEDULER,
        // VgaBuffer,
        SCREEN_BUFFER,
    },
    alloc::{
        format,
        string::{String, ToString},
        vec,
    },
    core::sync::atomic::{AtomicU64, Ordering::*},
    facepalm::start_facepalm,
    lazy_static::lazy_static,
    log::*,
};
lazy_static! {
    pub static ref TICK: AtomicU64 = AtomicU64::new(0);
    pub static ref BOOT_CONF: BootConfig = boot_conf::BootConfig::new();
}

/// The main entry point of the kernel
#[no_mangle]
pub fn kernel_main() -> ! {
    // log::set_max_level(BOOT_CONF.log_level());
    init::init();
    // info!("AbleOS {} {}", KERNEL_VERSION, RELEASE_TYPE);
    let result = logger::init();
    match result {
        Ok(_) => {}
        Err(err) => error!("{}", err),
    }
    log_version_data();

    let mut vterm0 = Vterm::new();
    vterm0.set_cursor_position(60, 40);
    vterm0.characters[0][0].character = 'a';

    //

    start_facepalm();
    sloop()
}

/// called by arch specific timers to tick up all kernel related functions
pub fn tick() {
    let mut data = TICK.load(Relaxed);
    data += 1;

    crate::kernel_state::KERNEL_STATE.lock().update_state();
    // let mut scheduler = SCHEDULER.lock();
    // scheduler.bump_exec();

    TICK.store(data, Relaxed)
}

pub fn cpu_socket_startup() {
    let mut cpu_info_socket = SimpleSock::new();
    cpu_info_socket.register_protocol("CPU_INFO".to_string());

    let x = master().unwrap();
    let _xyz = x.brand_string().unwrap();
}

pub fn log_version_data() {
    info!("{} v{}", RELEASE_TYPE, KERNEL_VERSION);
    info!(
        "Brand String: {}",
        master().unwrap().brand_string().unwrap()
    );
}

pub fn generate_process_pass() -> u128 {
    use rdrand::RdRand;
    let gen = RdRand::new().unwrap();
    let ret = (gen.try_next_u64().unwrap() as u128) << 64 | (gen.try_next_u64().unwrap() as u128);
    ret
}
