use super::*;

use crate::scheduler::{
    capabilities::{ControllerAccess, FileAccess, KeyboardAccess, MouseAccess, SoundCardAccess},
    proc::PID,
    Priority::*,
    Scheduler,
};

// #[test]
fn test_new_process() {
    let mut scheduler = Scheduler::new();
    scheduler.new_process(High);
    assert_eq!(scheduler.list.len(), 1);
}

// #[test]
fn test_next_process() {
    let mut scheduler = Scheduler::new();
    scheduler.new_process(High);
    scheduler.next_process();
    assert_eq!(scheduler.list.len(), 0);
}

// #[test]
fn test_term_process() {
    let mut scheduler = Scheduler::new();
    scheduler.new_process(High);
    scheduler.term_process(PID(1));
    assert_eq!(scheduler.list.len(), 0);
}

// #[test]
fn test_bump_exec() {
    let mut scheduler = Scheduler::new();
    scheduler.new_process(High);
    scheduler.bump_exec();
    assert_eq!(scheduler.process_exec_time, 1);
}

// #[test]
fn test_capabilities() {
    let caps = Capabilities::empty();
    assert_eq!(caps.files, FileAccess::None);
    assert_eq!(caps.mouse, MouseAccess::No);
    assert_eq!(caps.keyboard, KeyboardAccess::No);
    assert_eq!(caps.controllers, ControllerAccess::None);
    assert_eq!(caps.sound_cards, SoundCardAccess::None);
}
